@@ -2,8 +2,11 @@ package org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils;
 
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
 
 import org.gcube.common.homelibrary.home.HomeLibrary;
 import org.gcube.common.homelibrary.home.exceptions.InternalErrorException;
@@ -55,13 +58,16 @@ public class WorkspaceUtils {
 		copiedFolder.setDescription(bean.getDescription());
 
 		// change name of the copied folder to match the title (append the timestamp to avoid ties)
-		((WorkspaceFolder)copiedFolder).rename(org.gcube.datacatalogue.ckanutillibrary.utils.UtilMethods.fromProductTitleToName(bean.getTitle()) + "_" + System.currentTimeMillis());
+		long referenceTime = System.currentTimeMillis();
+		((WorkspaceFolder)copiedFolder).rename(org.gcube.datacatalogue.ckanutillibrary.utils.UtilMethods.fromProductTitleToName(bean.getTitle()) + "_" + referenceTime);
 
 		// copy only the selected ones
 		for(ResourceElementBean resource : resourcesToAdd){
 
 			if (resource.isToBeAdded()) {
 
+				logger.debug("Resource to add is " + resource);
+
 				// ok it is a file, so copy it into the copiedFolder
 				WorkspaceItem copiedFile = userCatalogue.addWorkspaceItem(resource.getOriginalIdInWorkspace(), copiedFolder.getId());
 
@@ -74,15 +80,13 @@ public class WorkspaceUtils {
 						copiedFile.getDescription(), 
 						copiedFile.getId(),
 						userName, 
-						null, // to be set
+						null, // dataset id, to be set
 						((FolderItem)copiedFile).getMimeType()));
 
 				// postpone rename operation
-				copiedFile.rename(resource.getName() + "_" + System.currentTimeMillis());
+				copiedFile.rename(resource.getName() + "_" + referenceTime);
 			}
 		}
-
-		// return
 		return resources;
 	}
 
@@ -113,7 +117,20 @@ public class WorkspaceUtils {
 
 		// Create the folder in the catalogue
 		Map<String, String> folderItems = Utils.getGcubeItemProperties(originalFolder);
-		bean.setCustomFields(folderItems);
+
+		if(folderItems != null){
+			// transform this properties
+			Map<String, List<String>> tempItems = new HashMap<String, List<String>>(folderItems.size());
+
+			Iterator<Entry<String, String>> iterator = folderItems.entrySet().iterator();
+			while (iterator.hasNext()) {
+				Map.Entry<java.lang.String, java.lang.String> entry = (Map.Entry<java.lang.String, java.lang.String>) iterator
+						.next();
+				tempItems.put(entry.getKey(), Arrays.asList(entry.getValue()));
+			}
+			
+			bean.setCustomFields(tempItems);
+		}
 
 		// set them into the bean
 		bean.setResources(Arrays.asList(WorkspaceUtils.getTreeFromFolder(folderId, ws)));
@@ -128,22 +145,23 @@ public class WorkspaceUtils {
 	 * @throws Exception 
 	 */
 	public static ResourceElementBean getTreeFromFolder(String workspaceFolderId, Workspace ws) throws Exception{
-
-		// start tree construction
 		ResourceElementBean rootElem = new ResourceElementBean();
 		String pathSeparator = ws.getPathSeparator();
 		try{
-			WorkspaceItem initialItemWS = ws.getItem(workspaceFolderId);
-			String fullPathBase = initialItemWS.getPath();
+			WorkspaceItem initialItem = ws.getItem(workspaceFolderId);
+			String fullPathBase = initialItem.getPath();
 			fullPathBase = fullPathBase.endsWith(ws.getPathSeparator()) ? fullPathBase : fullPathBase + ws.getPathSeparator();
-			rootElem.setFolder(initialItemWS.isFolder());
-			rootElem.setFullPath(initialItemWS.getPath().replace(fullPathBase, ""));
+			rootElem.setFolder(initialItem.isFolder());
+			rootElem.setFullPath(initialItem.getPath().replace(fullPathBase, ""));
 			rootElem.setParent(null);
-			rootElem.setName(initialItemWS.getName());
-			rootElem.setOriginalIdInWorkspace(initialItemWS.getId());
-			replaceFullPath(rootElem, pathSeparator);
-			// start visiting
-			visit(rootElem, initialItemWS, fullPathBase, pathSeparator);
+			rootElem.setName(initialItem.getName());
+			rootElem.setOriginalIdInWorkspace(initialItem.getId());
+			rootElem.setDescription(initialItem.getDescription());
+			extractEditableNameFromPath(rootElem, pathSeparator);
+
+			// recursive visiting
+			if(initialItem.isFolder())
+				visit(rootElem, initialItem, fullPathBase, pathSeparator);
 		}catch(Exception e){
 			logger.error("Failed to build the resource tree", e);
 			return null;
@@ -151,44 +169,47 @@ public class WorkspaceUtils {
 		return rootElem;
 	}
 
-	/**
-	 * Replaces the "/" char with a custom one
-	 * @param rootElem
-	 * @param pathSeparatorInWs
-	 */
-	private static void replaceFullPath(ResourceElementBean rootElem, String pathSeparatorInWs) {
-		
-		if(rootElem == null)
-			return;
-		
-		String elemName = rootElem.getName();
-		int lastIndex = rootElem.getFullPath().lastIndexOf(elemName);
-		String fullPath = rootElem.getFullPath().substring(0, lastIndex);
-		fullPath = fullPath.replaceAll(pathSeparatorInWs, RESOURCES_NAME_SEPARATOR) + elemName;
-		rootElem.setFullPath(fullPath);
-	}
-
 	/**
 	 * Recursive visit of a workspace item
 	 * @param rootElem
 	 * @param initialItemWS
 	 * @throws InternalErrorException
 	 */
-	private static void visit(ResourceElementBean rootElem, WorkspaceItem initialItemWS, String fullPathBase, String pathSeparator) throws InternalErrorException {
+	private static void visit(ResourceElementBean parent, WorkspaceItem initialItemWS, String fullPathBase, String pathSeparator) throws InternalErrorException {
 		List<? extends WorkspaceItem> children = initialItemWS.getChildren();
 		ArrayList<ResourceElementBean> childrenInTree = new ArrayList<ResourceElementBean>(children.size());
-		for (WorkspaceItem workspaceItem : children) {
+		for (WorkspaceItem item : children) {
 			ResourceElementBean elem = new ResourceElementBean();
-			elem.setFolder(workspaceItem.isFolder());
-			elem.setOriginalIdInWorkspace(workspaceItem.getId());
-			elem.setFullPath(workspaceItem.getPath().replace(fullPathBase, "")); 
-			elem.setParent(rootElem);
-			elem.setName(workspaceItem.getName());
-			replaceFullPath(elem, pathSeparator);
+			elem.setFolder(item.isFolder());
+			elem.setOriginalIdInWorkspace(item.getId());
+			elem.setFullPath(item.getPath().replace(fullPathBase, "")); 
+			elem.setParent(parent);
+			elem.setName(item.getName());
+			elem.setDescription(item.getDescription());
+			extractEditableNameFromPath(elem, pathSeparator);
 			childrenInTree.add(elem);
-			visit(elem, workspaceItem, fullPathBase, pathSeparator);
+			if(item.isFolder())
+				visit(elem, item, fullPathBase, pathSeparator);
 		}
-		rootElem.setChildren(childrenInTree);
+		// add these list as child of the rootElem
+		parent.setChildren(childrenInTree);
+	}
+
+	/**
+	 * Replaces the "/" char with a custom one and return an editable name for the user
+	 * @param rootElem
+	 * @param pathSeparatorInWs
+	 */
+	private static void extractEditableNameFromPath(ResourceElementBean rootElem, String pathSeparatorInWs) {
+
+		if(rootElem == null)
+			return;
+
+		String elemName = rootElem.getName();
+		int lastIndex = rootElem.getFullPath().lastIndexOf(elemName);
+		String fullPath = rootElem.getFullPath().substring(0, lastIndex);
+		fullPath = fullPath.replaceAll(pathSeparatorInWs, RESOURCES_NAME_SEPARATOR) + elemName;
+		rootElem.setEditableName(fullPath);
 	}
 
 }
